import axios, { isAxiosError } from 'axios';
export const getTimeout = () => {
    try {
        if (window) {
            return 20000;
        }
        if (process.env.NODE_ENV === 'test') {
            return 500;
        }
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
    }
    catch (e) {
        if (process.env.NODE_ENV === 'test') {
            return 500;
        }
    }
    return 20000;
};
export const getRetryTimeout = () => {
    try {
        if (window) {
            return 35000;
        }
        if (process.env.NODE_ENV === 'test') {
            return 4000;
        }
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
    }
    catch (e) {
        if (process.env.NODE_ENV === 'test') {
            return 4000;
        }
    }
    return 35000;
};
const ax = axios.create({
    baseURL: 'http://localhost:8080',
    timeout: getTimeout(),
});
let initialized = false;
export function resetInitialized() {
    if (process.env.NODE_ENV !== 'test') {
        throw new Error('resetInitialized can be called only in test environment');
    }
    initialized = false;
}
function shouldRetry(error, count, limit, started) {
    var _a;
    if (isAxiosError(error) && ((_a = error.response) === null || _a === void 0 ? void 0 : _a.status) === 404) {
        return false;
    }
    // console.log(
    //   'initialized',
    //   initialized,
    //   started,
    //   Date.now(),
    //   'diff',
    //   started && Date.now() - started,
    // );
    // retry for at least 35s if not yet initialized
    if (!initialized && started && Date.now() - started < getRetryTimeout()) {
        return true;
    }
    return !initialized && count < limit;
}
/**
 * Network refresh token storage handler.
 * Compliant with <strong><em>/api/setdata</em></strong> and <strong><em>/api/getdata</em></strong> endpoints of Kiosk Local Service.
 */
// @ts-ignore
export class NetworkRefreshTokenHandler {
    static setRefreshToken(tenant, refreshToken, count = 1) {
        return (ax
            .post('/api/setdata', {
            createdAt: Date.now(),
            updatedAt: Date.now(),
            name: `${tenant}_refresh_token`,
            value: refreshToken,
        })
            .then(() => {
            var _a;
            NetworkRefreshTokenHandler.debug &&
                ((_a = NetworkRefreshTokenHandler.AppInsights) === null || _a === void 0 ? void 0 : _a.trackTrace({
                    message: 'setRefreshToken success',
                    severityLevel: 0 /* eSeverityLevel.Verbose */,
                }, {
                    tenant,
                }));
            return;
        })
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            .catch((err) => {
            var _a;
            if (shouldRetry(err, count, 3)) {
                return NetworkRefreshTokenHandler.setRefreshToken(tenant, refreshToken, count + 1);
            }
            (_a = NetworkRefreshTokenHandler.AppInsights) === null || _a === void 0 ? void 0 : _a.trackTrace({
                message: 'setRefreshToken error',
                severityLevel: 3 /* eSeverityLevel.Error */,
            }, {
                tenant,
                error: err.message,
            });
            console.error('error saving refresh token', err);
            return;
        }));
    }
    static getRefreshToken(tenant, 
    // eslint-disable-next-line @typescript-eslint/no-empty-object-type
    options) {
        const started = (options === null || options === void 0 ? void 0 : options.started) || Date.now();
        if (options === null || options === void 0 ? void 0 : options.attemptCallback) {
            options.attemptCallback();
        }
        const count = (options === null || options === void 0 ? void 0 : options.count) || 1;
        return (ax
            .post('/api/getdata', { name: `${tenant}_refresh_token` })
            .then(({ data }) => {
            var _a;
            NetworkRefreshTokenHandler.debug &&
                ((_a = NetworkRefreshTokenHandler.AppInsights) === null || _a === void 0 ? void 0 : _a.trackTrace({
                    message: 'getRefreshToken success',
                    severityLevel: 0 /* eSeverityLevel.Verbose */,
                }, {
                    tenant,
                }));
            initialized = true;
            return data.value;
        })
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            .catch((err) => {
            var _a;
            if (shouldRetry(err, count, 10, started)) {
                return NetworkRefreshTokenHandler.getRefreshToken(tenant, {
                    count: count + 1,
                    attemptCallback: options === null || options === void 0 ? void 0 : options.attemptCallback,
                    started,
                });
            }
            (_a = NetworkRefreshTokenHandler.AppInsights) === null || _a === void 0 ? void 0 : _a.trackTrace({
                message: 'getRefreshToken error',
                severityLevel: 3 /* eSeverityLevel.Error */,
            }, {
                tenant,
                error: err.message,
            });
            // console.error('error getting refresh token', err);
            return '';
        }));
    }
    static clearRefreshToken(tenant, count = 1) {
        return (ax
            .post('/api/setdata', {
            createdAt: Date.now(),
            updatedAt: Date.now(),
            name: `${tenant}_refresh_token`,
            value: '',
        })
            .then(() => {
            var _a;
            NetworkRefreshTokenHandler.debug &&
                ((_a = NetworkRefreshTokenHandler.AppInsights) === null || _a === void 0 ? void 0 : _a.trackTrace({
                    message: 'clearRefreshToken success',
                    severityLevel: 0 /* eSeverityLevel.Verbose */,
                }, {
                    tenant,
                }));
            return;
        })
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            .catch((err) => {
            var _a;
            if (shouldRetry(err, count, 3)) {
                return NetworkRefreshTokenHandler.clearRefreshToken(tenant, count + 1);
            }
            (_a = NetworkRefreshTokenHandler.AppInsights) === null || _a === void 0 ? void 0 : _a.trackTrace({
                message: 'clearRefreshToken error',
                severityLevel: 3 /* eSeverityLevel.Error */,
            }, {
                tenant,
                error: err.message,
            });
            console.error('error removing refresh token', err.message, err);
            return;
        }));
    }
    static setAppInsights(appInsights) {
        NetworkRefreshTokenHandler.AppInsights = appInsights;
    }
    static setDebug(debug) {
        NetworkRefreshTokenHandler.debug = debug;
    }
}
NetworkRefreshTokenHandler.AppInsights = null;
NetworkRefreshTokenHandler.debug = false;
